
-- Test pattern check module for the Simplex G-LINK S-LINK LDC
-- To be used with Simplex G-LINK S-LINK LSC, Test Source Card (TSC) version  
-- Created on 11. Feb. 1999
-- Developer: Zoltan Meggyesi

-- LD (Data) bits in data mode (UTDO_N high)
-- 31,    30,  29,  28,  27,       26,   25..24
-- CAV_N, DAV, ERR, FLG, linkrdy_n; Gnd; URL[1.0],
-- 23,        22,          21,       20,                  19..0
-- Data err., Parity err., reserved, Data or Parity err., Data[19..0]

-- Returned Error bits
-- Data Comparison Error, Parity Error, Data Outside Frame  

-- URL    : URL1 URL0   
--          1    1   Walking 1  
--          1    0   Walking 0 
--          0    1   10101010
--          0    0   11110000

-- Component behav. models are generated by ACTEL macro generator
--====================================================================================
-- Modifications: 
-- 17.02.99: New PCB without bugs: HP_SMRST0 is on output pin 186, one less "G" output
-- 31.03.99: Only erroneous words are written with TP4 jumper, DERRLED_N Latches errors
-- 07.07.99: LD 27->HP_LINKRDY_N; LD26->Gnd; LD[25..24]->URL[1..0]
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------

LIBRARY IEEE;
USE     IEEE.STD_LOGIC_1164.ALL;
USE     IEEE.STD_LOGIC_ARITH.ALL;
USE     IEEE.STD_LOGIC_UNSIGNED.ALL;

entity check is

  port(
        CLK          : in    std_logic;  -- Clock signal
        WEN_N        : in    std_logic;  -- Write enable;
        RL1, RL0     : in    std_logic;
        TP4          : in    std_logic;  -- Write only erroneous words
        DATA         : in    std_logic_vector(31 downto 0); 
        LD           : out   std_logic_vector(31 downto 0);
        LWEN_N       : out   std_logic;  -- Write enable
        led_on       : out   std_logic); -- Error Led on

end check;

ARCHITECTURE behavioural OF check IS

component dload         -- Multiplexer 
    port( Data0 : in std_logic_vector(15 downto 0);
          Data1 : in std_logic_vector(15 downto 0);
          Data2 : in std_logic_vector(15 downto 0);
          Data3 : in std_logic_vector(15 downto 0);
          Sel0, Sel1 : in std_logic; 
          Result : out std_logic_vector(15 downto 0)) ;
end component;

component sshift        -- Shiftreg, shift left!
    port( Data : in std_logic_vector(15 downto 0);
          Enable, Shiften, Shiftin, Clock : in std_logic;
 	  Q : out std_logic_vector(15 downto 0)) ;
end component;

component parity        -- 4 input XOR
    port( Data : in std_logic_vector(3 downto 0);
          Result : out std_logic) ;
end component;

signal indat1, indat2, indat3, ldr: std_logic_vector(31 downto 0); -- Input pipeline
signal word0, word1, word2, word3, testdat: std_logic_vector(15 downto 0);  
signal shld, shift1out: std_logic_vector(15 downto 0);
signal frcnt: std_logic_vector(7 downto 0);
signal sof, shiften, shift1in: std_logic; -- Shiftreg signals
signal testpar, par: std_logic_vector(3 downto 0);  
signal start, derr, perr, write1_n, write2_n, write3_n, lwenr: std_logic;
signal ctrl1_n, ctrl2_n, ctrl3_n, length: std_logic;
signal led_in, led_out: std_logic; -- Data error led state machine

begin

-- Start of frame detection ------------------------------------------
  -- This process detects the EOF control word and generates a pulse
  -- This is used for synchronizing to the incoming data pattern
  -- DATA31=CAV_N, DATA30=DAV_N;

  eofdet: process (CLK) begin
   
    if (CLK'event and CLK='1') then
      if ((ctrl1_n or DATA(30)) = '0') then start <='1'; -- Shiftreg load
      else start <= '0';
      end if; 
    end if;
  
  end process eofdet;

-- Mux for pattern select ------------------------------------------------------
  
  ldata: dload port map (word0, word1, word2, word3, RL0, RL1, shld);

  word0 <= "0000000000000001"; -- Walking One
  word1 <= "1111111111111110"; -- Walking Zero
  word2 <= "1010101010101010"; -- Checkerboard
  word3 <= "1111111100000000";

-- Shift Register for pattern generation ---------------------------------------
  shiftreg: sshift port map (shld, start, shiften, shift1in, CLK, shift1out);

	shift1in <= shift1out(15); -- Cyclic
    shiften  <= '1';     -- Always enable

-- Parity generator -----------------------------------------------

  pargen3:  parity port map (indat2(15 downto 12), par(3)); 
  pargen2:  parity port map (indat2(11 downto 8),  par(2));
  pargen1:  parity port map (indat2(7  downto 4),  par(1));
  pargen0:  parity port map (indat2(3  downto 0),  par(0)); 
  
  -- Pipeline
  testdata: Process (CLK) begin
    if (CLK'event and CLK='1') then
    testpar  <= par;
    testdat  <= shift1out;
    end if;
  end process testdata;


-- Frame Length Counter ---------------------

  framecount:process(CLK) begin   
    if (CLK'event and CLK='1') then
      if (start='1') then frcnt <= (others => '0');
      else frcnt <= frcnt + '1';
      end if;
    end if;    
  end process framecount;
  
  framelength:process(CLK) begin   
    if (CLK'event and CLK='1') then
      if ((ctrl2_n or start)='0') then  -- True at end of frame  
        case frcnt is
          when "11100000" => length <= '0';   -- D"224"
          when others => length <= '1';
        end case;
      else length <= '0';
      end if;
    end if;
  end process framelength;

-- Comparator --------------------------------------------------------------------

  pipe: Process (CLK) begin  -- Delays inputs so as to sync. with test pattern 
    if (CLK'event and CLK='1') then
      indat1 <= DATA;
      indat2 <= indat1;
      indat3 <= indat2;

      write1_n <= WEN_N;
      write2_n <= write1_n;
      write3_n <= write2_n;

      ctrl1_n <= DATA(31);
      ctrl2_n <= ctrl1_n;
      ctrl3_n <= ctrl2_n;

    end if;
  end process pipe;

  dcompar: Process (CLK) begin
    if (CLK'event and CLK='1') then
      if  ((write3_n = '0') and 
         (indat3(15 downto 0) /= testdat)) then derr <='1'; -- If DATA <> testdata
      else derr <='0';
      end if;
    end if;
  end process dcompar;

  pcompar: Process (CLK) begin
    if (CLK'event and CLK='1') then
      if (((ctrl3_n and (not write3_n)) ='1') and -- Do not check Control word's parity
         (indat3(19 downto 16) /= testpar)) then perr <='1';
                                          -- If rx parity neq test par.
      else perr <='0';
      end if;
    end if;
  end process pcompar;

-- Data error LED ----------------------------------------------------------------
 led_on <= derr or perr or ldr(29); 

-- Output Data  ------------------------------------------------------------------
  outreg: Process (CLK) begin  -- Delays Data input 
    if (CLK'event and CLK='1') then
      ldr(31 downto 24) <= indat3(31 downto 24);
      ldr(19 downto 0)  <= indat3(19 downto 0);
      ldr(21) <= length; -- '1' if frame length error
      lwenr <= write3_n;     
    end if;
  end process outreg;

     ldr(23) <= derr;    
     ldr(22) <= perr;
     ldr(20) <= derr or perr or ldr(21);   -- par_err data_err or length-error

  ioreg: Process (CLK) begin  -- Delays Data input 
    if (CLK'event and CLK='1') then
      LD <= ldr;
      case TP4 is
        when '1'    => LWEN_N <= lwenr;  -- Jumper off
        when others => LWEN_N <= lwenr or (not(derr or perr or ldr(29) ));
        -- Write data/control words if comparison/parity or "HP" error occurred
      end case;

    end if;
  end process ioreg;
    
end behavioural;
     

  


