
-- Simplex G-LINK S-LINK LDC With test pattern check 
-- Created on 11. Feb. 1999
-- Developers: Zoltan Meggyesi & Erik van der Bij
-- Pinout and logic options in TDC.acf file
-- LD (Data) bits in data mode (UTDO_N high)
-- 31,    30,  29,  28,  27,           26,  25..24,    23..20,   19..0
-- CAV_N, DAV, ERR, FLG, HP_LINKRDY_N, Gnd, URL[1..0], Reserved, Data[19..0]
-- In 16 bit mode LD[19..16] is low
--====================================================================================
-- Jumpers: TP(n) signal is '0' when the TP(n) jumper is plugged (on) 
--          TP1 off: no flag;       on: flag mode
--          TP2 off: 16 bit mode;   on: 20 bit;
--          TP3 off: Locking "2";   on: Locking "3"
--          TP4 off: LWEN_N as LDC; on: Write only erroneous words

-- URL[1..0]selects test pattern: 
-- 0: Walking one; 1: Walking Zero; 2: 10101010; 3:11110000

--====================================================================================
-- Modifications: 
-- 17.02.99: New PCB without bugs: HP_SMRST0 is on output pin 186, one less "G" output
-- 31.03.99: Only erroneous words are written with TP4 jumper, DERRLED_N Latches errors
-- 19.05.99: Wen_n# signal (a source of LWEN_N) is also enabled when link is down
-- 07.07.99: LD 27->HP_LINKRDY_N; LD26->Gnd; LD[25..24]->URL[1..0]  Chksum: 96F0
--------------------------------------------------------------------------------------

LIBRARY IEEE;
USE     IEEE.STD_LOGIC_1164.ALL;
USE     IEEE.STD_LOGIC_ARITH.ALL;
USE     IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TDC is

  port(
        QCLK            : in    std_logic;  -- 40 Mhz from Quartz
        TP              : in    std_logic_vector(5 downto 1); -- Jumpers
        
        AD_OTI          : in    std_logic; 
        AD_DIO          : in    std_logic;

        HP_STRBOUT      : in    std_logic;  -- RX clock from HP
        HP_LINKRDY_N    : in    std_logic;  -- Low when link is active    
        HP_CAV_N        : in    std_logic;  -- G control word
        HP_DAV_N        : in    std_logic;  -- G data word
        HP_ERROR        : in    std_logic;  -- Error flag
        HP_FF           : in    std_logic;  -- Fill Frame
        HP_FLAG         : in    std_logic;
        HP_STAT1        : in    std_logic;  -- HP State Machine Output
        HP_STAT0        : in    std_logic;
        HP_D            : in    std_logic_vector(19 downto 0);

        URL             : in    std_logic_vector(3 downto 0);
        URESET_N        : in    std_logic;
        UXOFF_N         : in    std_logic;
        UTDO_N          : in    std_logic;
        UDW             : in    std_logic_vector(1 downto 0);

        fr_rxsigdet     : in    std_logic;
        fr_rxsys        : in    std_logic_vector(1 downto 0);

        AD_CONVST_N     : out   std_logic;
        AD_SCLK         : out   std_logic;
        AD_RDWR         : out   std_logic; 
		
        HP_EQEN         : out   std_logic; -- Cable equalization
        HP_FLAGSEL      : out   std_logic; -- Flag Mode Select
        HP_LOOPEN       : out   std_logic; -- Loopback control
        HP_M20SEL       : out   std_logic; -- 20/16 bit mode
        HP_SMRST1_N     : out   std_logic; -- State m. reset
        HP_SMRST0_N     : out   std_logic; -- State m. reset
        HP_TCLK         : out   std_logic; -- External test CLK
        HP_TCLKsel      : out   std_logic; -- Enable TCLK Input
        HP_DIV          : out   std_logic_vector(1 downto 0); -- Baud rate

        LD              : out   std_logic_vector(31 downto 0);
        LCLK            : out   std_logic; -- S-Link Signals 
        LCTRL_N         : out   std_logic;
        LWEN_N          : out   std_logic;
        LDOWN_N         : out   std_logic;
        LDERR_N         : out   std_logic;

        G               : out   std_logic_vector(25 downto 0); 
          -- Unused Pins are connected to ground to improvee gnd-bounce
        CK40TTL         : out   std_logic; 
        TESTLED_N       : out   std_logic; -- Leds, lights when 0
        DERRLED_N       : out   std_logic;
        LUPLED_N        : out   std_logic;
        FLOWCTLED_N     : out   std_logic);

end TDC;

ARCHITECTURE behavioural OF TDC IS

component check   -- Checks the test patterns
  port(CLK, WEN_N, RL1, RL0, TP4: in  std_logic;
       DATA: in std_logic_vector(31 downto 0);
       LD: out std_logic_vector(31 downto 0);
       LWEN_N, led_on: out std_logic);
end component;

TYPE   init_state  is (DOWN, UP, RES1, RES2, RES3,  LERR);
--attribute TYPE_ENCODING_STYLE of init_state:type is ONEHOT;
TYPE   reset_state is (OK, RSTNOW, WAITST);

signal curr_reset, next_reset: reset_state;  -- Reset machine
signal curr_init, next_init: init_state;     -- Link init machine
signal inreg: std_logic_vector(19 downto 0); -- Input register (.q)
signal rl: std_logic_vector(1 downto 0);  
signal wen_n, linkrdy_n, cav_n, dav_n: std_logic;
signal err, errled, flag, sys_res, led_on: std_logic; 
signal filt: std_logic_vector(10 downto 0);   -- HP_LINKRDY_N Filter    

signal cld: std_logic_vector(31 downto 0);  -- Data to checker

begin

-- Non-registered OUTPUTS -----------------------------------------------

HP_EQEN    <= '0';          -- Cable equalization disabled
HP_FLAGSEL <= not TP(1);    -- TP1 = vcc without jumper
HP_LOOPEN  <= (not HP_STAT1) and (not TP(3)); 
  -- Simplex Method 2 if TP3='1' (off), S.M 3 if TP3 is on  
HP_M20SEL  <= not TP(2);    -- 20 bit mode with TP2 jumper
HP_TCLK    <= '0';          -- Not used
HP_TCLKsel <= '0';          -- Test Clock input Disabled
HP_DIV     <= "00";  -- 700-1500 MBaud/sec

-- LCLK isInverted for correct timing, check Tsu/Th on Simulator!
LCLK       <= not HP_STRBOUT;  

LCTRL_N    <= '1';        -- No control words in this version
 
TESTLED_N  <= UTDO_N;     -- Will indicate laser/temp measurement

AD_CONVST_N  <= '1';      -- active low
AD_SCLK    <= '0';        -- don't use A/D device now
AD_RDWR    <= '1';        -- Read

-- gnd ouTPuts against gnd-bounce (page 532 in 1998 ALTERA data book)
G <=  (others => '0');

-- EDGE SENSITIVE RESET: Generates a pulse on negative URESET_N edge ----

reset_reg :process(HP_STRBOUT) begin
  if (HP_STRBOUT'event and HP_STRBOUT ='1') then
      curr_reset <= next_reset;
  end if;
end process reset_reg;

reset_machine :process(curr_reset, URESET_N) begin
   case curr_reset is
      when OK =>          -- No reset, URESET_N is high
         if (URESET_N ='0') then next_reset <= RSTNOW;
         else next_reset <= OK;
         end if;

      when RSTNOW =>      -- Reset pulse 
         next_reset <= WAITST;
         
      when WAITST =>     -- Wait while URESET_N is low; 
         if (URESET_N ='1') then next_reset <= OK;
         else next_reset <= WAITST;
         end if;

      when others => next_reset <= WAITST;
   end case;
end process reset_machine;

reset_out: process (curr_reset) begin
  case curr_reset is
      when RSTNOW => sys_res <= '1'; -- Active high system reset
      when others => sys_res <= '0';
  end case;
end process reset_out;

-- "Filtered" HPREADY (hpdown signal) ----------------------------
-- This signal has cca. 100ns wide negative pulses during 
-- synchronisation. Sync. with methode "3" @40mhz takes cca. 25ms
-- These "glitches are filtered with a counter

  filter:process(HP_STRBOUT) begin   
    if (HP_STRBOUT'event and HP_STRBOUT='1') then
      if (HP_LINKRDY_N = '1') then filt <= (others => '0');
      elsif (filt(10)='1') then filt <= filt;  -- stop counting if MSB=1
      else filt <= filt + '1';
      end if;
    end if;    -- filt(10) -> filtered act.high HP_READY
  end process filter;
-- LINK INIT STATE MACHINE -----------------------------------------

init_reg :process(HP_STRBOUT) begin
  if (HP_STRBOUT'event and HP_STRBOUT='1') then
  curr_init <= next_init;
end if;
end process init_reg;

init_machine: process (curr_init, sys_res) begin
 
case curr_init is 
    when DOWN =>               -- Wait for LINKRDY; Power-On state
        if (filt(10) = '1') then
          next_init <= UP;     -- Link UP if LINKRDY
        elsif (sys_res = '1') then 
          next_init <= RES1;   -- HP reset possible in DOWN state
        else next_init <= DOWN;
        end if;

    when UP =>                 -- when LINK UP
        if (sys_res = '1') then next_init <= RES1;
          elsif (filt(10) = '0') then next_init <= LERR; 
          else next_init <= UP;
        end if;

    when RES1 =>                -- HP Reset Pulse Here!
        next_init <= RES2;      -- As in S-LINK spec, 
   
    when RES2 =>                -- LDOWN Period for 4 LCLK 
         if (filt(10) = '0') then
           next_init <= RES3;   -- Wait fot HP-Down
           else NEXT_INIT <= RES2;
         end if;        
    
    when RES3 => 
         next_init <= DOWN;  
   
    when LERR =>               -- Link error, quit with URESET#
        case sys_res is
          when '1'    => next_init <= RES1;  -- Reset HP
          when others => next_init <= LERR;
        end case;
   
    when others =>             -- Same as LERR (Link Error)
        case sys_res is
          when '1'    => next_init <= RES1;
          when others => next_init <= LERR;
	    end case;
end case;
end process init_machine;


init_out: process (HP_STRBOUT) begin
if (HP_STRBOUT'event and HP_STRBOUT='1') then
   case curr_init is
     when UP =>     LDOWN_N     <= '1'; -- Link is active in UP state
                    LUPLED_N    <= '0';
                    HP_SMRST1_N <= '1';
     when RES1 =>   LDOWN_N     <= '0';
                    LUPLED_N    <= '1';
                    HP_SMRST1_N <= '0'; -- Reset HP Here!
     when others => LDOWN_N     <= '0';
                    LUPLED_N    <= '1';
                    HP_SMRST1_N <= '1';
     end case;
end if; 
end process init_out;

HP_SMRST0_N <= '1';  -- Not used

refclock: process (curr_init, TP(3)) begin
case TP(3) is
  when '1' => CK40TTL <= '0';   -- TP3 off, Lock Methode "2" 
  when others =>                -- TP3 on, Lock Methode "3"
    case curr_init is
      when UP =>     CK40TTL   <= '0';  -- Disabled when link is UP
      when others => CK40TTL   <= QCLK;
    end case;
end case;
end process refclock;

-- DATA ERROR and FLOWCTRL LEDS ---------------------------------

  -- Data error led must stay on while link is down because "HP Error"
  -- is active during this period so it would always go on

e_led: process (HP_STRBOUT) begin     
  if (HP_STRBOUT'event and HP_STRBOUT='1') then
    if (curr_init /= UP) then errled <= '1';
     -- Goes off on URESET# or LDOWN#
      elsif (led_on='1') then errled <= '0';  
      else errled <= errled;
    end if; 
  end if;
end process e_led;
DERRLED_N <= errled;

f_led: process (HP_STRBOUT) begin  
  if (HP_STRBOUT'event and HP_STRBOUT='1') then
    if (UXOFF_N = '0') then FLOWCTLED_N <= '0';   -- Goes on on UXOFF_N#
    else FLOWCTLED_N <= '1';
    end if;
  end if;
end process f_led;

-- INPUT REGISTER ----------------------------------------------

inputreg: process (HP_STRBOUT) begin
    if (HP_STRBOUT'event and HP_STRBOUT='1') then
      inreg(19 downto 0) <= HP_D(19 downto 0);
      dav_n  <= HP_DAV_N; 
      cav_n  <= HP_CAV_N; 
      err    <= HP_ERROR;
      flag   <= HP_FLAG;
      rl     <= URL(1 downto 0);
      linkrdy_n <= HP_LINKRDY_N;
    end if;
end process inputreg;    

-- OUTPUT REGISTER (pipeline to checker) ----------------------------------

outputreg: process (HP_STRBOUT) begin
  if (HP_STRBOUT'event and HP_STRBOUT='1') then
       
       cld(31) <= cav_n; cld(30) <= dav_n; cld(29) <= err; cld(28) <= flag;
       cld(27) <= linkrdy_n;
       cld(26) <= '0';   
       cld(25 downto 24) <= rl;
       cld(23 downto 20) <= ('0','0','0','0'); 
       cld(15 downto 0)  <= inreg(15 downto 0);
       
       case TP(2) is                                      -- not HP_M20SEL
         when '0'    => cld(19 downto 16) <= inreg(19 downto 16);  -- Jumper
         when others => cld(19 downto 16) <= ('0','0','0','0');    -- No jumper  
       end case;

-- wen_n# signal is active on DAV# and CAV# ----------------------
  -- LWEN_N# is generated from this signal in check module
  -- Was enabled only when (curr_init=UP) in previous version

      if ( (dav_n = '0') or (cav_n = '0')) -- and (curr_init = UP)
        then wen_n <= '0';
        else wen_n <= '1';            -- LWEN_N is active low
      end if;            
      
      LDERR_N <= '1';  --not err;     -- Not used in this version
  end if; 
  end process outputreg;

-- Data checker ----------------------------------------------------------

checklogic: check port map 
(HP_STRBOUT, wen_n, rl(1), rl(0), TP(4), cld,  LD, LWEN_N, led_on);

  end behavioural;
     

  


